from stnm.cli.common.process import get_node_process

from stnm.cli.common.response import error_response, success_response


def status():
    process = get_node_process()

    if process is None:
        error_response(2)

    success_response(1, **{"pid": process.pid})





DEFAULT_CONFIG = """
[node]
rpc_bind = "0.0.0.0:20443"
p2p_bind = "0.0.0.0:20444"
bootstrap_node = "048dd4f26101715853533dee005f0915375854fd5be73405f679c1917a5d4d16aaaf3c4c0d7a9c132a36b8c5fe1287f07dad8c910174d789eb24bdfb5ae26f5f27@krypton.blockstack.org:20444"

[burnchain]
chain = "bitcoin"
mode = "krypton"
peer_host = "bitcoind.krypton.blockstack.org"
rpc_port = 18443
peer_port = 18444
process_exit_at_block_height = 5130

[[mstx_balance]]
address = "STB44HYPYAT2BB2QE513NSP81HTMYWBJP02HPGK6"
amount = 10000000000000000

[[mstx_balance]]
address = "ST11NJTTKGVT6D1HY4NJRVQWMQM7TVAR091EJ8P2Y"
amount = 10000000000000000

[[mstx_balance]]
address = "ST1HB1T8WRNBYB0Y3T7WXZS38NKKPTBR3EG9EPJKR"
amount = 10000000000000000

[[mstx_balance]]
address = "STRYYQQ9M8KAF4NS7WNZQYY59X93XEKR31JP64CP"
amount = 10000000000000000
"""


import shutil
import subprocess
from typing import Optional

import psutil


def get_executable(name: str) -> Optional[psutil.Process]:
    return shutil.which(name)


def get_node_process() -> Optional[psutil.Process]:
    try:
        pid = subprocess.check_output(["pgrep", "-f", "stacks-node"]).decode()
    except subprocess.CalledProcessError:
        return None

    return psutil.Process(int(pid))



import os
import subprocess

from stnm.cli.common.process import get_node_process
from stnm.cli.common.response import error_response, success_response
from stnm.helper.config import get_config_path, get_config_parsed, put_config_parsed


def start():
    process = get_node_process()
    if process is not None:
        error_response(3)

    config = get_config_parsed()
    if "__modified__" in config:
        del config["__modified__"]
        put_config_parsed(config)

    devnull = open(os.devnull, 'wb')
    process = subprocess.Popen(["stacks-node", "start", "--config", get_config_path()], stdout=devnull, stderr=devnull)
    success_response(3, **{"pid": process.pid})



import re
from typing import Union, List

from stnm.cli.common.response import error_response, success_response
from stnm.helper.config import get_config_path, get_config, get_config_parsed, put_config_parsed

"""
Config Input Structure

        +---------------------+
        |      parameter      |
        +---------------------+
input = burnchain.burn_fee_cap=20000000
        |section |     key    | value |
        +-----------------------------+
        |        config entry         |
        +-----------------------------+
        
"""


class InvalidConfigEntry(Exception):
    pass


class UnavailableConfigParameter(Exception):
    pass


class InvalidValue(Exception):
    pass


def bool_validator(s: str) -> bool:
    if s in ["true", "True"]:
        return True

    if s in ["false", "False"]:
        return False

    raise InvalidValue("{} is not a valid boolean value. Use True or true or False or false".format(s))


def str_validator(s: str) -> str:
    return s


def int_validator(s: str):
    if re.compile("^[0-9]+$").match(s):
        return int(s)

    raise InvalidValue("{} is not a valid integer value".format(s))


AVAILABLE_PARAMS = {
    "node.miner": bool_validator,
    "node.seed": str_validator,
    "burnchain.burn_fee_cap": int_validator,
    "burnchain.process_exit_at_block_height": int_validator,
    "burnchain.rpc_port": int_validator,
    "burnchain.peer_port": int_validator
}


class ConfigEntry:
    def __init__(self, section: str, key: str, value: Union[str, int, bool]):
        self.section = section
        self.key = key
        self.value = value

    def __repr__(self):
        return "<{}.{}={}>".format(self.section, self.key, self.value)


def config_input_parser(input_: str) -> List[ConfigEntry]:
    rv = []
    entries = input_.split(",")

    for entry in entries:
        param_split = entry.split("=")
        if len(param_split) != 2:
            raise InvalidConfigEntry(entry)

        [param, raw_value] = param_split
        if param not in AVAILABLE_PARAMS:
            raise UnavailableConfigParameter(param)

        value = AVAILABLE_PARAMS[param](raw_value)

        [section, key] = param.split(".")

        rv.append(ConfigEntry(section, key, value))

    return rv


def config(input_: str):
    if input_ == "show":
        config_contents = get_config()
        print("-" * 60)
        print("Config file located at: {}".format(get_config_path()))
        print("-" * 60)
        print(config_contents)
        print("-" * 60)
        return

    try:
        entries = config_input_parser(input_)
    except InvalidConfigEntry as ex:
        error_response(5, str(ex))
        return
    except UnavailableConfigParameter as ex:
        error_response(6, str(ex))
        return
    except InvalidValue as ex:
        error_response(7, str(ex))
        return

    parsed = get_config_parsed()

    for e in entries:
        parsed[e.section][e.key] = e.value

    parsed["__modified__"] = True

    put_config_parsed(parsed)

    success_response(4)



import json
import subprocess
from typing import Dict

from flask import Flask, jsonify, request, abort

from stnm.helper.config import get_config_parsed

app = Flask(__name__)


def communicate(cmd: str, arg: str = "") -> Dict:
    process = subprocess.Popen(["stnm", cmd, arg], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    resp = process.communicate()[0].decode("utf-8")
    return json.loads(resp)


@app.route("/api", methods=["GET"])
def api_index():
    return jsonify({"hello": "world"})


@app.route("/api/status", methods=["GET"])
def api_status():
    return jsonify(communicate("status"))


@app.route("/api/start", methods=["POST"])
def api_start():
    return jsonify(communicate("start"))


@app.route("/api/stop", methods=["POST"])
def api_stop():
    return jsonify(communicate("stop"))


@app.route("/api/config", methods=["GET"])
def api_config_get():
    return jsonify(get_config_parsed())


@app.route("/api/config", methods=["POST"])
def api_config_post():
    data = request.json

    if type(data) == dict and "input" in data and type(data["input"]) == str and len(data["input"]) > 0:
        return jsonify(communicate("config", data["input"]))

    abort(400)


@app.route("/ui", methods=["GET"])
def ui_index():
    return "coming soon..."



import os

from stnm.cli.common.constants import DEFAULT_CONFIG
from stnm.cli.common.response import error_response
from stnm.cli.common.util import which
from stnm.cli.config import config
from stnm.cli.install import install
from stnm.cli.start import start
from stnm.cli.status import status
from stnm.cli.stop import stop
from stnm.helper.config import get_config_path
from stnm.test import do_tests
from stnm.web import web


def main(cmd: str, arg: str):
    if cmd == "install":
        install()

    # check node binary
    if which("stacks-node") is None:
        error_response(1)

    # check / create default config file
    if not os.path.isfile(get_config_path()):
        with open(get_config_path(), "w") as f:
            f.write(DEFAULT_CONFIG)
            f.close()

    if cmd == "status":
        status()

    if cmd == "start":
        start()

    if cmd == "stop":
        stop()

    if cmd == "config":
        config(arg)

    if cmd == "web":
        web()

    if cmd == "test":
        do_tests()



import sys

from setuptools import find_packages
from setuptools import setup

assert sys.version_info[0] == 3 and sys.version_info[1] >= 5, "Requires Python 3.5 or newer"

setup(
    name="stnm",
    version="0.1.2",
    description="Stacks blockchain node process manager",
    long_description="Stacks blockchain node process manager. See https://github.com/talhasch/stnm for mode details.",
    packages=find_packages(exclude=("venv",)),
    install_requires=[
        "psutil==5.7.3",
        "toml==0.10.2",
        "tornado==6.1",
        "Flask==1.1.2"
    ],
    entry_points={
        "console_scripts": [
            "stnm=stnm.run:main"
        ]
    })


import os
from pathlib import Path
from typing import Dict

import toml


def get_config_path() -> str:
    return os.path.abspath(os.path.join(str(Path.home()), "stnm.conf"))


def get_config() -> str:
    with open(get_config_path(), "r") as f:
        config_contents = f.read()
        f.close()

    return config_contents


def get_config_parsed() -> Dict:
    with open(get_config_path(), "r") as f:
        config_contents = f.read()
        f.close()

    return toml.loads(config_contents)


def put_config_parsed(obj: Dict):
    with open(get_config_path(), "w") as f:
        f.write(toml.dumps(obj))
        f.close()
